@@ -1,17 +1,16 @@
 pub mod account;
+pub mod backend;
 pub mod email;
 pub mod envelope;
 pub mod flag;
 pub mod folder;
-#[cfg(feature = "imap-backend")]
-pub mod imap;
+pub mod sender;
 pub mod tpl;
 
 pub use self::account::{args, handlers, Account, Accounts};
+pub use self::backend::*;
 pub use self::email::*;
 pub use self::envelope::*;
 pub use self::flag::*;
 pub use self::folder::*;
-#[cfg(feature = "imap-backend")]
-pub use self::imap::*;
 pub use self::tpl::*;
