@@ -2,7 +2,7 @@ use anyhow::{Context, Result};
 use log::debug;
 use pimalaya_email::{
     email::{local_draft_path, remove_local_draft},
-    AccountConfig, Backend, CompilerBuilder, Flag, Flags, Sender, Tpl,
+    AccountConfig, Backend, Flag, Flags, Sender, Tpl,
 };
 use std::{env, fs, process::Command};
 
@@ -37,11 +37,11 @@ pub fn open_with_local_draft() -> Result<Tpl> {
     open_with_tpl(Tpl::from(content))
 }
 
-pub fn edit_tpl_with_editor<P: Printer, B: Backend + ?Sized, S: Sender + ?Sized>(
+pub fn edit_tpl_with_editor<P: Printer>(
     config: &AccountConfig,
     printer: &mut P,
-    backend: &mut B,
-    sender: &mut S,
+    backend: &mut dyn Backend,
+    sender: &mut dyn Sender,
     mut tpl: Tpl,
 ) -> Result<()> {
     let draft = local_draft_path();
@@ -73,15 +73,17 @@ pub fn edit_tpl_with_editor<P: Printer, B: Backend + ?Sized, S: Sender + ?Sized>
         match choice::post_edit() {
             Ok(PostEditChoice::Send) => {
                 printer.print_log("Sending email…")?;
-                let email = tpl.compile(
-                    CompilerBuilder::default()
-                        .some_pgp_sign_cmd(config.email_writing_sign_cmd.as_ref())
-                        .some_pgp_encrypt_cmd(config.email_writing_encrypt_cmd.as_ref()),
-                )?;
+                let email = tpl
+                    .some_pgp_sign_cmd(config.email_writing_sign_cmd.clone())
+                    .some_pgp_encrypt_cmd(config.email_writing_encrypt_cmd.clone())
+                    .compile()?
+                    .write_to_vec()?;
                 sender.send(&email)?;
-                let sent_folder = config.sent_folder_alias()?;
-                printer.print_log(format!("Adding email to the {} folder…", sent_folder))?;
-                backend.add_email(&sent_folder, &email, &Flags::from_iter([Flag::Seen]))?;
+                if config.email_sending_save_copy {
+                    let sent_folder = config.sent_folder_alias()?;
+                    printer.print_log(format!("Adding email to the {} folder…", sent_folder))?;
+                    backend.add_email(&sent_folder, &email, &Flags::from_iter([Flag::Seen]))?;
+                }
                 remove_local_draft()?;
                 printer.print("Done!")?;
                 break;
@@ -96,11 +98,11 @@ pub fn edit_tpl_with_editor<P: Printer, B: Backend + ?Sized, S: Sender + ?Sized>
             }
             Ok(PostEditChoice::RemoteDraft) => {
                 let draft_folder = config.folder_alias("drafts")?;
-                let email = tpl.compile(
-                    CompilerBuilder::default()
-                        .some_pgp_sign_cmd(config.email_writing_sign_cmd.as_ref())
-                        .some_pgp_encrypt_cmd(config.email_writing_encrypt_cmd.as_ref()),
-                )?;
+                let email = tpl
+                    .some_pgp_sign_cmd(config.email_writing_sign_cmd.clone())
+                    .some_pgp_encrypt_cmd(config.email_writing_encrypt_cmd.clone())
+                    .compile()?
+                    .write_to_vec()?;
                 backend.add_email(
                     &draft_folder,
                     &email,
